var fs  = require('fs'),
    path = require('path')
    ex = require("fs-extra");

var encoding = 'binary';
var ROOT = process.cwd() + '/';
var buffer = null;
var msgs = [];
var types = ['success', 'fail', 'warn'];

var DOCUMENT_ROOT = "/";
var DOCUMENT_VERSION = "v1.0";
var DOCUMENT_TARGET_STATIC = "/";

var RESOURCES_ROOT = "/";
var RESOURCES_STATIC = "/";
var RESOURCES_TEMPLATES = "/";

var TEMP_RESOURCES_ROOT = "/";
var TEMP_RESOURCES_STATIC = "/";
var TEMP_RESOURCES_TEMPLATES = "/";

var mkdir = function(dest) {
    var destDirs = dest.split(/[\\/]/),
        destDirsLen = destDirs.length,
        destDir = '';

    if (!fs.existsSync(dest)) {
        destDirs.forEach(function(dir, i) {
            if (i < destDirsLen - 1) {
                destDir += dir + '/';
                if (!fs.existsSync(destDir)) {
                    fs.mkdirSync(destDir);
                    msgs.push({msg: '[mkdir] ' + destDir + ''});
                }
            }
        });
    }
};

var fixPath = function(src) {
    if (src && src.charAt(src.length - 1) !== '/') src += '/';
    return src;
};

var insertThymeleafNS = function(file){
    var thymeleafNS = 'xmlns:th="http://www.thymeleaf.org"';
    var pattern = /<html([^<>]+)>/gmi;

    pattern.lastIndex = 0;

    if(file.indexOf(thymeleafNS) == -1){
        // console.log("insertThymeleafNS")
        file = file.replace(pattern, "<html " + thymeleafNS + "$1>");
    }

    pattern = null;

    return file;
};

var parseInclude = function(content){
    //<!--#include virtual="/static/v1.0/inc/css_common_gm.html" -->
    var pattern = /<!\-\-[\s]*#include[\s]+virtual=[\"\'](\/?([^\"\']+)\.([a-z]+))[\"\'][\s]*\-\->/gmi;

    pattern.lastIndex = 0;

    content = content.replace(pattern, '<th:block th:include="~{$2 :: thymeleaf}" />');

    pattern = null;
    return content;
};

var replaceAttrInc = function(content){
    var pattern = /<!\-\-[\s]*#include[\s]+virtual=[\"\'](\/?([^\"\']+attr\.html))[\"\'][\s]*\-\->/gmi;
    var matches = content.match(pattern) || [];

    //TEMP_RESOURCES_TEMPLATES
    matches.forEach(function(inc, i) {
        var incFile = inc.match(/"(.+)"\s*-->/i)[1];
        var incSrc = path.resolve(TEMP_RESOURCES_TEMPLATES + incFile);

        var attrContent = fs.readFileSync(incSrc, encoding);

        // console.log("inc==============")
        // console.log(inc);
        // console.log('============================')
        // console.log(attrContent)
        
        content = content.replace(inc, attrContent);
    });

    return content;
};

var parseExternalLink = function(content){
    var pattern = /(([\s]+)(href|src)([\s]*=[\s]*["'])([^\s\"\']+)(["']))/gmi;
    var matcher = null;
    pattern.lastIndex = 0;

    while(null != (matcher = pattern.exec(content))){
        if(matcher[5] && (matcher[5].charAt(0) != "#" && matcher[5].charAt(0) != "<" && !/^([^\/]+\:)?\/\//i.test(matcher[5]))){
            content = content.replace(matcher[1], matcher[2] + "th:" + matcher[3] + matcher[4] + "@{" + matcher[5] + "}" + matcher[6]);
        }
    }
    // content = content.replace(pattern, "$1th:$2$3@{$4}$5");

    pattern = null;
    return content;
};

var toThymeleaf = function(filepath){
    var file = fs.readFileSync(filepath, encoding);

    file = replaceAttrInc(file);
    file = insertThymeleafNS(file);
    // file = parseExternalLink(file);
    file = parseInclude(file);

    if(filepath.indexOf("/inc/") != -1){
        file = '<template th:fragment="thymeleaf">' + file + '</template>'
    }

    fs.writeFileSync(filepath, file, encoding);
};

var transform = function(from){
    console.log("transform::" + from);
    if (fs.statSync(from).isFile()) {
        toThymeleaf(from);
    }
    else if (fs.statSync(from).isDirectory()) {
        from = fixPath(from);

        var files = fs.readdirSync(from);
        var fileSrc = null;

        files.forEach(function(src, i) {
            fileSrc = from + src;

            if (fs.statSync(fileSrc).isFile()) {
                toThymeleaf(fileSrc);
            } else {
                transform(fileSrc);
            }
        });
    }
};


var shtml2thymeleaf = function(callback) {
    buffer = {};

    var from = TEMP_RESOURCES_TEMPLATES + DOCUMENT_TARGET_STATIC;

    transform(from);

    //同步到正式目录，并删除临时目录
    ex.copySync(TEMP_RESOURCES_TEMPLATES, RESOURCES_TEMPLATES, {
        overwrite: true
    });
    ex.removeSync(TEMP_RESOURCES_ROOT);

    callback(msgs);

    buffer = null;
    msgs = [];
};

var rename = function(src, callback){
    var _baseDir = src;
    if (fs.statSync(src).isFile()) {
        fs.renameSync(src, src.replace(/\.shtml$/i, ".html"));
        console.log("s0::" + src);
    }
    else if (fs.statSync(src).isDirectory()) {
        src = fixPath(src);

        var files = fs.readdirSync(src);
        var targetFile = null;

        files.forEach(function(file, i) {
            targetFile = src + file;

            if (fs.statSync(targetFile).isFile()) {
                if (path.extname(file) !== '.shtml') return;
                
                fs.renameSync(targetFile, targetFile.replace(/\.shtml$/i, ".html"));

                console.log("s1::" + targetFile);
            } else {
                rename(targetFile, callback);
                console.log("s2::" + targetFile);
            }
        });
    }
};

var copy = function(from, to, callback){
    var copyFrom = DOCUMENT_ROOT + DOCUMENT_TARGET_STATIC + from;
    var copyTo = to + DOCUMENT_TARGET_STATIC;

    mkdir(copyTo);

    if(fs.existsSync(copyFrom)){
        ex.copySync(copyFrom, copyTo + from, {
            overwrite: true
        });
        msgs.push({msg: '[' + copyFrom + '] sync success `' + copyTo + '`', type: types[0]});
    }else{
        msgs.push({msg: '[' + copyFrom + '] directory not existed `' + copyTo + '`', type: types[2]});
    }
}

var syncResources = function(callback){
    copy("src", RESOURCES_STATIC, callback, false);
    copy("res", RESOURCES_STATIC, callback, false);
    copy("fonts", RESOURCES_STATIC, callback, false);
    copy("media", RESOURCES_STATIC, callback, false);
    copy("sw", RESOURCES_STATIC, callback, false)

    copy("html", TEMP_RESOURCES_TEMPLATES, callback, true);
    copy("inc", TEMP_RESOURCES_TEMPLATES, callback, true);

    rename(TEMP_RESOURCES_TEMPLATES, callback);
};

var clearResources = function(){
    ex.removeSync(RESOURCES_ROOT);
};


var se2thai = function(wwwroot, target, callback){
    if (!wwwroot) {
        msgs.push({msg: '`wwwroot` not set, use the current directory(' + ROOT + ')', type: types[2]});
        wwwroot = ROOT;
    }
    if (!target) {
        msgs.push({msg: '`target` not set, use default static version(v1.0)', type: types[2]});
        target = "v1.0";
    }

    callback(msgs);
    msgs = [];

    DOCUMENT_ROOT = fixPath(wwwroot);
    DOCUMENT_VERSION = target
    DOCUMENT_TARGET_STATIC = fixPath("static/" + DOCUMENT_VERSION);

    RESOURCES_ROOT = DOCUMENT_ROOT + "resources/";
    RESOURCES_STATIC = RESOURCES_ROOT + "static/";
    RESOURCES_TEMPLATES = RESOURCES_ROOT + "templates/";

    TEMP_RESOURCES_ROOT = DOCUMENT_ROOT + "_resources/";
    TEMP_RESOURCES_STATIC = TEMP_RESOURCES_ROOT + "static/";
    TEMP_RESOURCES_TEMPLATES = TEMP_RESOURCES_ROOT + "templates/";

    clearResources();
    syncResources(callback);
    shtml2thymeleaf(callback);
}


module.exports = se2thai;
